package org.kantega.jexmec;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * This interface defines a strategy for the discovery and removal of
 * plugin class loaders. Provider implementations may add class loader on start and remove them on stop or
 * they may add and remove class loaders from spawned threads.
 */
public interface PluginClassLoaderProvider {
    /**
     * Start the PluginClassLoaderProvider.
     *
     * @param registry a registry where the provider can add and remove ClassLoaders
     * @param parentClassLoader the class loader to use as parent for class loaders created by this provider
     */
    void start(Registry registry, ClassLoader parentClassLoader);

    /**
     * Stop the PluginClassLoaderProvider. Implementations must remove any currently registered class loaders.
     */
    void stop();

    /**
     * A registry where providers can add and remove class loaders. Providers who need to remove and add class loaders
     * with a single call to {@link PluginManagerListener#pluginsUpdated(java.util.List)} may do so using the
     * {@link #replace(Iterable, Iterable)} method.
     */
    public interface Registry {

        /**
         * Add class loaders to the system
         *
         * @param classLoaders the class loaders to add
         */
        <T extends ClassLoader> void add(Iterable<T> classLoaders);


        /**
         * Removes class loaders from the system.
         *
         * @param classLoaders the classloader to remove
         */
        <T extends ClassLoader> void remove(Iterable<T> classLoaders);

        /**
         * Removed and adds classloaders in a single batch. The removal and addition will be result in
         * {@link PluginManagerListener#pluginsUpdated(java.util.List)} being called only once.
         * @param removeClassLoaders class loaders to remove
         * @param addClassLoaders the class loaders to add
         */
        <T extends ClassLoader> void replace(Iterable<T> removeClassLoaders, Iterable<T> addClassLoaders);

    }

}